@@ -12,18 +12,19 @@ from hdroller.die.base import BaseDie
 from hdroller.die.scalar import ScalarDie
 from hdroller.die.vector import VectorDie
 
-min = hdroller.die.base.BaseDie.min
-max = hdroller.die.base.BaseDie.max
-min_outcome = hdroller.die.base.BaseDie.min_outcome
+highest = hdroller.die.base.BaseDie.highest
+lowest = hdroller.die.base.BaseDie.lowest
 max_outcome = hdroller.die.base.BaseDie.max_outcome
+min_outcome = hdroller.die.base.BaseDie.min_outcome
 
-from hdroller.pool import Pool, DicePool
-from hdroller.eval_pool import EvalPool, SumPool, sum_pool, FindMatchingSets
+from hdroller.pool import Pool, standard_pool, DicePool
+from hdroller.eval_pool import EvalPool, WrapFuncEval, SumPool, sum_pool, FindBestSet, FindBestRun
 
 __all__ = ['Die', 'standard', 'd', '__getattr__', 'bernoulli', 'coin',
     'BaseDie', 'ScalarDie', 'VectorDie',
     'from_cweights', 'from_sweights', 'from_rv', 'mix', 'if_else', 'align', 'align_range', 'check_ndim',
-    'min', 'max', 'min_outcome', 'max_outcome',
+    'lowest', 'highest', 'max_outcome', 'min_outcome',
     'apply',
-    'Pool', 'DicePool', 'EvalPool', 'SumPool', 'sum_pool', 'FindMatchingSets',
+    'Pool', 'standard_pool', 'DicePool',
+    'EvalPool', 'WrapFuncEval', 'SumPool', 'sum_pool', 'FindBestSet', 'FindBestRun',
     'd2', 'd3', 'd4', 'd6', 'd8', 'd10', 'd12', 'd20', 'd100']
