@@ -17,7 +17,8 @@ def Pool(die, num_dice=None, count_dice=None, *, max_outcomes=None, min_outcomes
     
     You can use `die.pool(args...)` for the same effect as this function.
     
-    All instances are cached.
+    All instances are cached. The members of the actual instance may not match the arguments exactly;
+    instead, they may be optimized to values that give the same result as far as `EvalPool` is concerned.
     
     This is capitalized because it is the preferred way of getting a new instance,
     and so that you can use `from hdroller import Pool` while leaving the name `pool` free.
@@ -25,9 +26,9 @@ def Pool(die, num_dice=None, count_dice=None, *, max_outcomes=None, min_outcomes
     
     Args:
         die: The fundamental die of the pool.
-            Note that this will **not** be clipped to `min_outcomes` or `max_outcomes`
-            even if no die in the pool can actually reach the extreme outcomes of the fundamental die.
-            `PoolEval` will see the unrollable outcomes as being rolled 0 times each.
+            If outcomes are not reachable by any die due to `min_outcomes` or `max_outcomes`,
+            they will have 0 count. Zero-weight outcomes will appear with zero weight,
+            but can still generate nonzero counts.
         num_dice: An `int` that sets the number of dice in the pool.
             If no arguments are provided, this defaults to 0.
         count_dice: Determines which of the **sorted** dice will be counted, and how many times each.
@@ -42,6 +43,7 @@ def Pool(die, num_dice=None, count_dice=None, *, max_outcomes=None, min_outcomes
             `[-1, 1]` would roll two dice, counting the higher die as a positive and the lower die as a negative.
             
             You can also use the `[]` operator to select dice from an existing pool.
+            For example, `pool[-2:]` would also count the two highest dice.
             This is always an absolute selection on all `num_dice`,
             not a relative selection on already-selected dice,
             which would be ambiguous in the presence of multiple or negative counts.
@@ -50,18 +52,19 @@ def Pool(die, num_dice=None, count_dice=None, *, max_outcomes=None, min_outcomes
             provided that neither `min_outcomes` nor `max_outcomes` are set.
             For example, you could create a pool of 4d6 drop lowest using `hdroller.d6.pool()[0, 1, 1, 1]`.
         max_outcomes: A sequence of one outcome per die in the pool.
-            That die will be limited to that maximum outcome, with all higher outcomes being removed (i.e. rerolled).
-            Values above the `max_outcome` of the fundamental die have no effect.
+            That die will be limited to that maximum outcome, with all higher outcomes having 0 weight.
+            Values cannot be > the `max_outcome` of the fundamental die.
             A pool cannot limit both `min_outcomes` and `max_outcomes`.
             This can be used to efficiently roll a set of mixed standard dice.
             For example, `Pool(hdroller.d12, max_outcomes=[6, 6, 6, 8, 8])` would be a pool of 3d6 and 2d8.
         min_outcomes: A sequence of one outcome per die in the pool.
-            That die will be limited to that minimum outcome, with all lower outcomes being removed (i.e. rerolled).
-            Values below the `min_outcome` of the fundamental die have no effect.
+            That die will be limited to that minimum outcome, with all lower outcomes having 0 weight.
+            Values cannot be < the `min_outcome` of the fundamental die.
             A pool cannot limit both `min_outcomes` and `max_outcomes`.
     
     Raises:
-        `ValueError` if arguments conflict with each other.
+        `ValueError` if arguments result in a conflicting number of dice, 
+            or if `max_outcome` or `min_outcome` fall outside the range of the die.
     """
     
     # Compute num_dice.
@@ -94,13 +97,19 @@ def Pool(die, num_dice=None, count_dice=None, *, max_outcomes=None, min_outcomes
     else:
         if max_outcomes is not None:
             if min(max_outcomes) < die.max_outcome():
-                max_outcomes = tuple(sorted(min(outcome, die.max_outcome()) for outcome in max_outcomes))
+                if max(max_outcomes) > die.max_outcome():
+                    raise ValueError('max_outcomes cannot be > the max_outcome of the die.')
+                # We can't trim the die to max_outcomes since it may upset the iteration order.
+                max_outcomes = tuple(sorted(die.nearest_le(outcome) for outcome in max_outcomes))
             else:
                 # In this case, the max_outcomes don't actually do anything.
                 max_outcomes = None
         if min_outcomes is not None:
             if max(min_outcomes) > die.min_outcome():
-                min_outcomes = tuple(sorted(max(outcome, die.min_outcome()) for outcome in min_outcomes))
+                if min(min_outcomes) < die.min_outcome():
+                    raise ValueError('min_outcomes cannot be < the min_outcome of the die.')
+                # We can't trim the die to min_outcomes since it may upset the iteration order.
+                min_outcomes = tuple(sorted(die.nearest_ge(outcome) for outcome in min_outcomes))
             else:
                 # In this case, the min_outcomes don't actually do anything.
                 min_outcomes = None
@@ -136,6 +145,20 @@ def _pool_cached_unchecked(die, count_dice, max_outcomes, min_outcomes):
     """
     return DicePool(die, count_dice, max_outcomes=max_outcomes, min_outcomes=min_outcomes)
 
+def standard_pool(*die_sizes, count_dice=None):
+    """ Creates a pool of standard dice.
+    
+    For example, `standard_pool(8, 8, 6, 6, 6)` would be a pool of 2 d8s and 3 d6s.
+    
+    Args:
+        *die_sizes: The size of each die in the pool.
+        count_dice: Which dice will be counted, as `Pool()`.
+            As with `Pool()`, you can also use the `[]` operator after the fact.
+            For example, `standard_pool(8, 8, 6, 6, 6)[-2:]` would keep the highest two dice
+            of 2 d8s and 3 d6s.
+    """
+    return Pool(hdroller.d(max(die_sizes)), count_dice=count_dice, max_outcomes=die_sizes)
+
 class DicePool():
     def __init__(self, die, count_dice, *, max_outcomes, min_outcomes):
         """ Unchecked constructor.
@@ -160,6 +183,28 @@ class DicePool():
     def count_dice(self):
         """ A tuple indicating how many times each of the dice, sorted from lowest to highest, counts. """
         return self._count_dice
+    
+    @cached_property
+    def _num_drop_lowest(self):
+        for i, count in enumerate(self.count_dice()):
+            if count:
+                return i
+        return self.num_dice()
+    
+    def num_drop_lowest(self):
+        """ How many elements of count_dice on the low side have a false truth value. """
+        return self._num_drop_lowest
+    
+    @cached_property
+    def _num_drop_highest(self):
+        for i, count in enumerate(reversed(self.count_dice())):
+            if count:
+                return i
+        return self.num_dice()
+    
+    def num_drop_highest(self):
+        """ How many elements of count_dice on the high side have a false truth value. """
+        return self._num_drop_highest
         
     def __getitem__(self, count_dice):
         """ Returns a pool with the selected dice counted, as the `count_dice` argument to `Pool()`.
@@ -193,8 +238,6 @@ class DicePool():
     def num_dice(self):
         return len(self._count_dice)
     
-    __len__ = num_dice
-    
     def max_outcomes(self, always_tuple=False):
         """ A tuple of sorted max outcomes, one for each die in the pool. 
         
@@ -229,52 +272,45 @@ class DicePool():
             * weight: An `int` indicating the weight of that many dice rolling the removed outcome.
         """
         max_outcomes = self.max_outcomes(always_tuple=True)
-        remaining_count = sum(self.count_dice())
-
         num_possible_dice = self.num_dice() - bisect.bisect_left(max_outcomes, self.die().max_outcome())
         num_unused_dice = self.num_dice() - num_possible_dice
         popped_die, outcome, single_weight = self.die().pop_max()
         
-        if len(popped_die) == 0:
+        if popped_die.is_empty():
             # This is the last outcome. All dice must roll this outcome.
-            weight = single_weight ** num_possible_dice
-            yield None, remaining_count, weight
-            return
-        
-        if popped_die.total_weight() == 0:
-            # This is the last outcome with positive weight. All dice must roll this outcome.
-            weight = single_weight ** num_possible_dice
             pool = Pool(popped_die, num_dice=0)
+            remaining_count = sum(self.count_dice())
+            weight = single_weight ** num_possible_dice
             yield pool, remaining_count, weight
             return
         
-        if not any(self.count_dice()):
-            # No selected dice remain. All dice must roll somewhere below, so empty all dice in one go.
-            # We could follow the staircase of max_outcomes more closely but this is unlikely to be relevant in most cases.
-            pool = Pool(popped_die, num_dice=0)
-            weight = math.prod(self.die().weight_le(max_outcome) for max_outcome in max_outcomes)
-            yield pool, 0, weight
-            return
-        
+        # Consider various numbers of dice rolling this outcome.
         popped_max_outcomes = max_outcomes[:num_unused_dice] + (popped_die.max_outcome(),) * num_possible_dice
         popped_count_dice = self.count_dice()
-        
-        # Zero dice rolling this outcome.
-        # If there is no weight, this is the only possibility.
-        pool = Pool(popped_die, count_dice=popped_count_dice, max_outcomes=popped_max_outcomes)
-        weight = 1
         count = 0
-        yield pool, count, weight
         
-        if single_weight > 0:
-            # If the outcome has nonzero weight, consider how many dice could roll this outcome.
-            comb_row = hdroller.math.comb_row(num_possible_dice, single_weight)
-            for weight in comb_row[1:]:
-                count += popped_count_dice[-1]
-                popped_max_outcomes = popped_max_outcomes[:-1]
-                popped_count_dice = popped_count_dice[:-1]
-                pool = Pool(popped_die, count_dice=popped_count_dice, max_outcomes=popped_max_outcomes)
-                yield pool, count, weight
+        comb_row = hdroller.math.comb_row(num_possible_dice, single_weight)
+        end_counted = self.num_dice() - self.num_drop_lowest()
+        for weight in comb_row[:min(num_possible_dice, end_counted)]:
+            pool = Pool(popped_die, count_dice=popped_count_dice, max_outcomes=popped_max_outcomes)
+            yield pool, count, weight
+            count += popped_count_dice[-1]
+            popped_max_outcomes = popped_max_outcomes[:-1]
+            popped_count_dice = popped_count_dice[:-1]
+        
+        if end_counted > num_possible_dice:
+            pool = Pool(popped_die, count_dice=popped_count_dice, max_outcomes=popped_max_outcomes)
+            yield pool, count, comb_row[-1]
+        else:
+            # In this case, we ran out of counted dice before running out of dice that could roll the outcome.
+            # We empty the rest of the pool immediately since no more dice can contribute counts.
+            skip_weight = 0
+            for weight in comb_row[end_counted:]:
+                skip_weight *= popped_die.total_weight()
+                skip_weight += weight
+            skip_weight *= math.prod(popped_die.weight_le(max_outcome) for max_outcome in max_outcomes[:num_unused_dice])
+            pool = Pool(popped_die, 0)
+            yield pool, count, skip_weight
     
     def _iter_pop_min(self):
         """
@@ -292,51 +328,44 @@ class DicePool():
         # However, the alternative of reversing the storage order of die_counts and min_outcomes seems even worse.
         
         min_outcomes = self.min_outcomes(always_tuple=True)
-        remaining_count = sum(self.count_dice())
-
         num_possible_dice = bisect.bisect_right(min_outcomes, self.die().min_outcome())
         popped_die, outcome, single_weight = self.die().pop_min()
         
-        if len(popped_die) == 0:
+        if popped_die.is_empty():
             # This is the last outcome. All dice must roll this outcome.
-            weight = single_weight ** num_possible_dice
-            yield None, remaining_count, weight
-            return
-        
-        if popped_die.total_weight() == 0:
-            # This is the last outcome with positive weight. All dice must roll this outcome.
-            weight = single_weight ** num_possible_dice
             pool = Pool(popped_die, num_dice=0)
+            remaining_count = sum(self.count_dice())
+            weight = single_weight ** num_possible_dice
             yield pool, remaining_count, weight
             return
         
-        if not any(self.count_dice()):
-            # No selected dice remain. All dice must roll somewhere below, so empty all dice in one go.
-            # We could follow the staircase of max_outcomes more closely but this is unlikely to be relevant in most cases.
-            pool = Pool(popped_die, num_dice=0)
-            weight = math.prod(self.die().weight_ge(min_outcome) for min_outcome in min_outcomes)
-            yield pool, 0, weight
-            return
-        
+        # Consider various numbers of dice rolling this outcome.
         popped_min_outcomes = (popped_die.min_outcome(),) * num_possible_dice + min_outcomes[num_possible_dice:]
         popped_count_dice = self.count_dice()
-        
-        # Zero dice rolling this outcome.
-        # If there is no weight, this is the only possibility.
-        pool = Pool(popped_die, count_dice=popped_count_dice, min_outcomes=popped_min_outcomes)
-        weight = 1
         count = 0
-        yield pool, count, weight
         
-        if single_weight > 0:
-            # If the outcome has nonzero weight, consider how many dice could roll this outcome.
-            comb_row = hdroller.math.comb_row(num_possible_dice, single_weight)
-            for weight in comb_row[1:]:
-                count += popped_count_dice[0]
-                popped_min_outcomes = popped_min_outcomes[1:]
-                popped_count_dice = popped_count_dice[1:]
-                pool = Pool(popped_die, count_dice=popped_count_dice, min_outcomes=popped_min_outcomes)
-                yield pool, count, weight
+        comb_row = hdroller.math.comb_row(num_possible_dice, single_weight)
+        end_counted = self.num_dice() - self.num_drop_highest()
+        for weight in comb_row[:min(num_possible_dice, end_counted)]:
+            pool = Pool(popped_die, count_dice=popped_count_dice, min_outcomes=popped_min_outcomes)
+            yield pool, count, weight
+            count += popped_count_dice[0]
+            popped_min_outcomes = popped_min_outcomes[1:]
+            popped_count_dice = popped_count_dice[1:]
+        
+        if end_counted > num_possible_dice:
+            pool = Pool(popped_die, count_dice=popped_count_dice, min_outcomes=popped_min_outcomes)
+            yield pool, count, comb_row[-1]
+        else:
+            # In this case, we ran out of counted dice before running out of dice that could roll the outcome.
+            # We empty the rest of the pool immediately since no more dice can contribute counts.
+            skip_weight = 0
+            for weight in comb_row[end_counted:]:
+                skip_weight *= popped_die.total_weight()
+                skip_weight += weight
+            skip_weight *= math.prod(popped_die.weight_ge(min_outcome) for min_outcome in min_outcomes[num_possible_dice:])
+            pool = Pool(popped_die, 0)
+            yield pool, count, skip_weight
     
     @cached_property
     def _pop_max(self):
@@ -362,6 +391,30 @@ class DicePool():
         """
         return self._pop_min
         
+    def has_counted_dice(self):
+        """ Returns `True` iff any of the remaining dice are counted a nonzero number of times.
+        
+        This is used to skip to the base case when there are no more dice to consider.
+        """
+        return any(self.count_dice())
+    
+    def eval(self, eval_or_func, /):
+        """ Evaluates this pool using the given `EvalPool` or function.
+        
+        Note that each `EvalPool` instance carries its own cache;
+        if you plan to use an evaluation multiple times,
+        you may want to explicitly create an `EvalPool` instance
+        rather than passing a function to this method directly.
+        
+        Args:
+            func: This can be an `EvalPool`, in which case it evaluates the pool directly.
+                Or it can be a `next_state()`-like function, taking in state, outcome, *counts and returning the next state.
+                In this case a temporary `WrapFuncEval` is constructed and used to evaluate this pool.
+        """
+        if not isinstance(eval_or_func, hdroller.EvalPool):
+            eval_or_func = hdroller.WrapFuncEval(eval_or_func)
+        return eval_or_func.eval(self)
+    
     def sum(self):
         """ Convenience method to simply sum the dice in this pool.
         
